trace = TracePoint.new(:end) do |tp|
  not_me = tp.self.ancestors.dup
  not_me.delete(tp.self)
  if not_me.include?(Interface)
    missing_methods = []
    Interface.abstract_methods.each do |m|
      unless tp.self.instance_methods.include?(m)
        missing_methods << m
      end
    end
    if missing_methods.any?
      raise Interface::NotImplementedError.new(*missing_methods)
    end
  end
end

trace.enable

module Interface

  class NotImplementedError < StandardError
    def initialize(*methods)
      super "You must implement the following methods: #{methods.join(', ')}"
    end
  end

  class << self

    attr_accessor :abstract_methods

    def abstract_method(*args)
      self.abstract_methods = args
    end

  end

  abstract_method :bar, :baz

end

class Foo
  include Interface

  def bar

  end

  def baz

  end
end